#include <config.h>

#include <gnome.h>
#include "gnome-ucombo.h"

#include "acc_entry.h"

/* this function is needed by GCompletion */
gchar *acc_entry_get_account_name(AccountData *account)
{
     return account->name;
}

/* this function is responsible for completion i case PgDn is pressed */
gint acc_entry_key_press_event(GtkWidget *entry, GdkEventKey *key_event, gpointer callback_data)
{
     GCompletion *completion;
     gchar *new_prefix;

     g_return_val_if_fail(entry != NULL, FALSE);
     g_return_val_if_fail(key_event != NULL, FALSE);

     if (key_event->keyval != GDK_Page_Down)
	  return FALSE;

     completion = g_completion_new((GCompletionFunc) acc_entry_get_account_name);
     g_completion_add_items(completion, acc_list);
     g_completion_complete(completion, 
			   gtk_entry_get_text(GTK_ENTRY(entry)), 
			   &new_prefix);

     gtk_entry_set_text(GTK_ENTRY(entry), new_prefix);

     g_free(new_prefix);
     g_completion_free(completion);

     return TRUE;
}

/* this function sets optimal width of the ctree widget, it is called whenever the tree expands/collapses */
void acc_entry_set_optimal_width(GtkCTree *ctree, GtkCTreeNode *node, gpointer callback_data)
{
     gint h = gtk_clist_optimal_column_width(GTK_CLIST(ctree), 0);
     g_print("test %d\n",h);
     gtk_clist_set_column_width(GTK_CLIST(ctree), 0, gtk_clist_optimal_column_width(GTK_CLIST(ctree), 0));
}

void acc_entry_test(GtkCTree *ctree, GtkCTreeExpansionType expansion, gpointer callback_data)
{
     gint h = gtk_clist_optimal_column_width(GTK_CLIST(ctree), 0);
     g_print("hejhej %d\n",h);
}

GtkCTreeNode *acc_entry_build_account_node(GtkWidget *ctree, AccountData * account, GtkCTreeNode *root)
{
    if (account->extra_data == NULL) {
	gchar *text[1];
	text[0] = account->name;

	if (account->parent == 0)
	    account->extra_data =
		gtk_ctree_insert_node(GTK_CTREE(ctree),
				      root, NULL,
			   text, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
	else
	    account->extra_data =
		gtk_ctree_insert_node(GTK_CTREE(ctree),
			       acc_entry_build_account_node(ctree, account->parent, root), NULL,
			   text, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
    }
    return GTK_CTREE_NODE(account->extra_data);
}

GtkWidget *acc_entry_build_ctree(const gboolean allow_none)
{
    GtkWidget *ctree;
    GtkCTreeNode *root;
    GList *cursor;

    gchar *titles[1] = {"Name"};
    ctree = gtk_ctree_new_with_titles(1, 0, titles);
    gtk_clist_column_titles_hide(GTK_CLIST(ctree));

    /* add the additional 'none' node */
    if (allow_none) {
	    gchar *text[1] = { _("<none>") };
	    root = gtk_ctree_insert_node(GTK_CTREE(ctree),
					 NULL, NULL,
					 text, 0, NULL, NULL, NULL, NULL, FALSE, TRUE);
    } else
	    root = NULL;

    /* we will use AccountData.extra_data as pointer to tree nodes - we have to set them to NULL first */
    for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
	AccountPtr(cursor->data)->extra_data = NULL;

    /* build all tree nodes */
    for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor)) {
	acc_entry_build_account_node(ctree, AccountPtr(cursor->data), root);
	gtk_ctree_node_set_row_data(GTK_CTREE(ctree),
				    GTK_CTREE_NODE(AccountPtr(cursor->data)->extra_data),
				    AccountPtr(cursor->data)->name);
    }
    gtk_ctree_sort_recursive(GTK_CTREE(ctree), NULL);

    /* set optimal (that is maximum that can happen) width of the ctree column widget */
    gtk_ctree_expand_recursive(GTK_CTREE(ctree), NULL);
    gtk_clist_set_column_width(GTK_CLIST(ctree), 0, gtk_clist_optimal_column_width(GTK_CLIST(ctree), 0));
    gtk_ctree_collapse_recursive(GTK_CTREE(ctree), NULL);
    /* actually I could do auto_resize, but then the dropdownbox moves to the left (why?) 
       gtk_clist_set_column_auto_resize(GTK_CLIST(ctree), 0, TRUE); */

    gtk_widget_show(ctree);

    return ctree;
}

void acc_entry_selected_from_tree(GtkWidget *ctree, GtkCTreeNode *node, gint column, GnomeUcombo *acc_entry)
{
	if (gtk_ctree_node_get_row_data(GTK_CTREE(ctree), node))
		gtk_entry_set_text(GTK_ENTRY(acc_entry->entry), 
				   (gchar *) gtk_ctree_node_get_row_data(GTK_CTREE(ctree), node));
	else
		gtk_entry_set_text(GTK_ENTRY(acc_entry->entry), _("<none>"));

	gnome_ucombo_hide_popup(acc_entry);
}

GtkWidget *acc_entry_new(const gboolean with_none)
{
	GtkWidget *acc_entry;

	GtkWidget *ctree;

	ctree = acc_entry_build_ctree(with_none);
	acc_entry = gnome_ucombo_new(ctree);
	gtk_signal_connect(GTK_OBJECT(GNOME_UCOMBO(acc_entry)->entry), "key_press_event", 
			   GTK_SIGNAL_FUNC(acc_entry_key_press_event), NULL);

	gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row", acc_entry_selected_from_tree, acc_entry);

	return acc_entry;
}

gint acc_entry_get_account(GtkWidget *acc_entry, AccountData **ret)
{
	GList *cursor;

	g_assert(GNOME_IS_UCOMBO(acc_entry));

	if (strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(acc_entry)->entry)), _("<none>")) == 0) {
	     *ret = NULL; /* the caller have to check if he likes the null value returned */
	     return 0;
	}

	for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
		if (!strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(acc_entry)->entry)), AccountPtr(cursor->data)->name)) {
			*ret = AccountPtr(cursor->data);
			return 0;
		}

	return 1; /* no matching account found */
}
