#include <config.h>

#include "chart_bar.h"

#include "chart.h"


GtkWidget *chart_canvas;
GnomeCanvasGroup *chart_root;

void chart_init()
{
     chart_acc_list = NULL;
     chart_group_list = NULL;
}

void chart_done()
{
     GList *cursor;

     for (cursor = chart_acc_list; cursor != NULL; cursor = g_list_next(cursor))
	  g_free(ChartGroupDataPtr(cursor->data));
     g_list_free(chart_acc_list);

     for (cursor = chart_group_list; cursor != NULL; cursor = g_list_next(cursor))
	  g_free(ChartGroupDataPtr(cursor->data));
     g_list_free(chart_group_list);
}

GtkWidget *chart_new()
{
     GtkWidget *scrolled_window;
     GnomeCanvasItem *no_data;

     chart_canvas = gnome_canvas_new();
     gtk_widget_set_usize(chart_canvas, CHART_CANVAS_SIZE_X / 2, CHART_CANVAS_SIZE_Y / 2);
     gnome_canvas_set_scroll_region(GNOME_CANVAS(chart_canvas), -10.0, -10.0, 
				    CHART_CANVAS_SIZE_X + 10.0, CHART_CANVAS_SIZE_Y + 10.0);

     chart_root = GNOME_CANVAS_GROUP(gnome_canvas_item_new(gnome_canvas_root (GNOME_CANVAS(chart_canvas)),
							   gnome_canvas_group_get_type(),
							   "x", (double) 0.0,
							   "y", (double) 0.0,
							   NULL));
     
     no_data = gnome_canvas_item_new(chart_root,
				     gnome_canvas_text_get_type(),
				     "text", "No chart data set - try >Chart->Chart Setup< first",
				     "anchor", GTK_ANCHOR_CENTER,
				     "font", "-*-fixed-medium-r-semicondensed--12-*-*-*-c-*-iso8859-1",
				     "x", (double) CHART_CANVAS_SIZE_X / 2.0,
				     "y", (double) CHART_CANVAS_SIZE_Y / 2.0,
				     "fill_color", "black",
				     NULL);

     gtk_widget_show(chart_canvas);

     scrolled_window = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     gtk_container_add(GTK_CONTAINER(scrolled_window), chart_canvas);
     gtk_widget_show(scrolled_window);

     return scrolled_window;
}

gdouble **chart_table_new(void)
{
     gint accounts_no, groups_no, i;
     gdouble **chart_table;

     accounts_no = g_list_length(chart_acc_list);
     groups_no = g_list_length(chart_group_list);

     chart_table = g_new(gdouble *, groups_no);
     for (i = 0; i < groups_no; i++)
	  chart_table[i] = g_new0(gdouble, accounts_no);
     
     return chart_table;
}

void chart_table_free(gdouble **chart_table)
{
     gint groups_no, i;
     groups_no = g_list_length(chart_group_list);
     for (i = 0; i < groups_no; i++)
	  g_free(chart_table[i]);
     g_free(chart_table);
}

void chart_table_calculate(gdouble **chart_table)
{
     GList *trans_cursor, *acc_cursor, *gr_cursor;
     gint i, j;

     for (gr_cursor = chart_group_list, i = 0; gr_cursor != NULL; gr_cursor = g_list_next(gr_cursor), i++) {

	  for (acc_cursor = acc_list; acc_cursor != NULL; acc_cursor = g_list_next(acc_cursor))
	       AccountPtr(acc_cursor->data)->amount = 0.0;

	  py_module_compile_str(ChartGroupDataPtr(gr_cursor->data)->expr);
	  for (trans_cursor = trans_list; trans_cursor != NULL; trans_cursor = g_list_next(trans_cursor)) {
	       TransactionData *transaction = TransactionPtr(trans_cursor->data);
	       AccountData *account;
	       gboolean chart_inbound;

	       py_module_set_trans(transaction);
	       chart_inbound = transaction->inbound && py_module_eval_code();

	       for (account = transaction->acc_from; account != NULL; account = account->parent)
		    if (chart_inbound || account->unlimited) {
			 if (account->reversed)
			      account->amount += transaction->amount;
			 else
			      account->amount -= transaction->amount;
		    }

	       for (account = transaction->acc_to; account != NULL; account = account->parent)
		    if (chart_inbound || account->unlimited) {
			 if (account->reversed)
			      account->amount -= transaction->amount;
			 else
			      account->amount += transaction->amount;
		    }
	  }
	  
	  for (acc_cursor = chart_acc_list, j = 0; acc_cursor != NULL; acc_cursor = g_list_next(acc_cursor), j++)
	       chart_table[i][j] = ChartAccountDataPtr(acc_cursor->data)->account->amount;
     }

     /* restore proper values of accounts */
     data_recalculate_accounts();
}

void chart_build(void)
{
     gdouble **chart_table;

     chart_table = chart_table_new();
     chart_table_calculate(chart_table);

     gtk_object_destroy(GTK_OBJECT(chart_root));
     chart_root = GNOME_CANVAS_GROUP(gnome_canvas_item_new(gnome_canvas_root (GNOME_CANVAS(chart_canvas)),
							   gnome_canvas_group_get_type(),
							   "x", (double) 0.0,
							   "y", (double) 0.0,
							   NULL));
     
     chart_bar_build(chart_root, chart_table);

     chart_table_free(chart_table);
}





