
#include <config.h>

#include <gnome.h>

#include "chart.h"

#include "chart_bar.h"


struct _ChartBar {
     GnomeCanvasGroup *root_canvas;

     gdouble base_line;
     gdouble scale;
} chart_bar;

void chart_bar_get_min_max(gdouble **chart_table, gdouble *min, gdouble *max)
{
     gint groups_no, accounts_no;
     gint i, j;

     accounts_no = g_list_length(chart_acc_list);
     groups_no = g_list_length(chart_group_list);

     *min = G_MAXDOUBLE;
     *max = G_MINDOUBLE;

     /* I know that it could be 25% faster, but it's not necessary, right? */
     for (i = 0; i < groups_no; i++)
	  for (j = 0; j < accounts_no; j++) {
	       *min = MIN(chart_table[i][j], *min);
	       *max = MAX(chart_table[i][j], *max);
	  }

     *max = MAX(0, *max);
     *min = MIN(0, *min);
}

void chart_bar_build_account(GnomeCanvasGroup *chart_root, 
			     gdouble x0, gdouble y0, gdouble account_width, gdouble account_height, 
			     gdouble **chart_table,
			     gint group_id, gint account_id,
			     gdouble min, gdouble max)
{
     const gdouble amount_text_width = 30;
     const gdouble account_vert_spacing = 1;

     if (group_id != (-1)) {
	  GnomeCanvasPoints *base_line_points;
	  gchar amount_text[32];
	  gdouble amount;
	  gdouble bar_length;
	  gdouble base_line;

	  amount = chart_table[group_id][account_id];
	  sprintf(amount_text, " %.2lf ", amount);

	  bar_length = amount * ((account_width - amount_text_width * 2.0) / (max - min));
	  base_line =  account_width - amount_text_width - 
	       (account_width - amount_text_width * 2.0) * (max / (max - min));

	  gnome_canvas_item_new(chart_root,
				gnome_canvas_rect_get_type(),
				bar_length < 0 ? "x1" : "x2", (double) x0 + base_line + bar_length,
				"y1", (double) y0 + account_vert_spacing,
				bar_length < 0 ? "x2" : "x1", (double) x0 + base_line,
				"y2", (double) y0 + account_height - account_vert_spacing,
				"fill_color", "RGB:87/DD/EB",
				"outline_color", "black",
				"width_pixels", 1,
				NULL);

	  gnome_canvas_item_new(chart_root,
				gnome_canvas_text_get_type(),
				"text", amount_text,
				"anchor", bar_length < 0 ? GTK_ANCHOR_E : GTK_ANCHOR_W,
				"font", "-*-fixed-medium-r-semicondensed-*-13-*-*-*-c-*-iso8859-1",
				"x", (double) x0 + base_line + bar_length,
				"y", (double) y0 + account_height / 2.0,
				"fill_color", "black",
				NULL);

	  base_line_points = gnome_canvas_points_new(2);
	  base_line_points->coords[0] = x0 + base_line;
	  base_line_points->coords[1] = y0;
	  base_line_points->coords[2] = base_line_points->coords[0];
	  base_line_points->coords[3] = y0 + account_height;
	  gnome_canvas_item_new(chart_root,
				gnome_canvas_line_get_type(),
				"points", base_line_points,
				"fill_color", "black",
				"width_pixels", 1,
				NULL);
     } else {
	  gnome_canvas_item_new(chart_root,
			gnome_canvas_text_get_type(),
			"text", ChartAccountDataPtr(g_list_nth(chart_acc_list, account_id)->data)->account->name,
			"anchor", GTK_ANCHOR_CENTER,
			"font", "-*-fixed-medium-r-semicondensed-*-13-*-*-*-c-*-iso8859-1",
			"x", (double) x0 + account_width / 2.0,
			"y", (double) y0 + account_height / 2.0,
			"fill_color", "black",
			NULL);
     }
}

void chart_bar_build_group(GnomeCanvasGroup *chart_root, 
			   gdouble x0, gdouble y0, gdouble group_width, gdouble group_height, 
			   gdouble **chart_table,
			   gint group_id, gdouble min, gdouble max)
{
     const gdouble title_height = 30;
     const gdouble border_width = 20;

     gdouble account_width, account_height;
     gint account_id, accounts_no;

     accounts_no = g_list_length(chart_acc_list);

     account_width = group_width - border_width;
     account_height = (group_height - border_width - title_height) / accounts_no;

     for (account_id = 0; account_id < accounts_no; account_id++)
	  chart_bar_build_account(chart_root,
				  x0 + border_width / 2.0, 
				  y0 + account_id * account_height + title_height + border_width / 2.0,
				  account_width, account_height,
				  chart_table,
				  group_id, account_id,
				  min, max);

     if (group_id != (-1))
	  gnome_canvas_item_new(chart_root,
				gnome_canvas_text_get_type(),
				"text", ChartGroupDataPtr(g_list_nth(chart_group_list, group_id)->data)->name,
				"anchor", GTK_ANCHOR_CENTER,
				"font", "-*-fixed-medium-r-semicondensed--12-*-*-*-c-*-iso8859-1",
				"x", (double) x0 + group_width / 2.0,
				"y", (double) y0 + title_height / 2.0,
				"fill_color", "black",
				NULL);
}

void chart_bar_build(GnomeCanvasGroup *chart_root, gdouble **chart_table)
{
     const gdouble legend_width = 80;
     const gdouble border_width = 10;

     gdouble group_width, group_height;
     gdouble min, max;
     gint group_id, groups_no;

     chart_bar_get_min_max(chart_table, &min, &max);
     groups_no = g_list_length(chart_group_list);

     group_width = (CHART_CANVAS_SIZE_X - border_width - legend_width) / groups_no;
     group_height = CHART_CANVAS_SIZE_Y - border_width;

     for (group_id = 0; group_id < groups_no; group_id++)
	  chart_bar_build_group(chart_root, 
				group_id * group_width + border_width / 2.0 + legend_width, border_width / 2.0, 
				group_width, group_height,
				chart_table,
				group_id, min, max);

     chart_bar_build_group(chart_root, 
			   0, border_width / 2.0, legend_width, group_height,
			   NULL,
			   -1, 0, 0);
}


