#include <config.h>

#include <gnome.h>

#include "chart.h"
#include "acc_entry.h"
#include "limit_dlg.h"

#include <stdlib.h> /* rand() and friends */

#include "chart_setup.h"


struct _ChartSetup {
     GtkWidget *dialog;

     GtkWidget *bar_chart;

     GtkWidget *account_entry;
     GtkWidget *account_list;
     GtkWidget *group_name;
     GtkWidget *group_expr;
     GtkWidget *group_list;
} chart_setup;

guint32 chart_setup_random_rgb(void)
{
     return rand() & 0x00FFFFFF;
}

GdkPixmap *chart_setup_color_pixmap(GtkWidget *widget, guint8 red, guint8 green, guint8 blue)
{
    GdkPixmap *pixmap;
    GdkColor color;
    GdkGC *gc;

    color.red = 255;
    color.green = 0;
    color.blue = 0;
		
    /*gdk_colormap_alloc_color(gtk_object_get_data(GTK_OBJECT(widget), "colormap"), &color, FALSE, TRUE);*/
    gdk_color_alloc(gdk_colormap_get_system(), &color);
    
    pixmap = gdk_pixmap_new(widget->window,
			    48, 16, gtk_widget_get_visual(widget)->depth);
	
    gc = gdk_gc_new(widget->window);
    gdk_gc_set_foreground(gc, &color);
    gdk_draw_rectangle(pixmap, gc, TRUE, 0, 0, 47, 15);
    gdk_gc_destroy(gc);
    
    return pixmap;
}

void chart_setup_add_account(GtkWidget *widget, gpointer callback_data)
{
     ChartAccountData *chart_account_data;
     AccountData *account;
     gchar *text[2];
     gint row;

     if (acc_entry_get_account(chart_setup.account_entry, &account) || (account == NULL))
	  return;

     chart_account_data = g_new(ChartAccountData, 1);
     chart_account_data->account = account;
     chart_account_data->rgb = chart_setup_random_rgb();

     text[0] = chart_account_data->account->name;
     text[1] = "";
     row = gtk_clist_append(GTK_CLIST(chart_setup.account_list), text);
     gtk_clist_set_row_data(GTK_CLIST(chart_setup.account_list), row, chart_account_data);
}

void chart_setup_add_direct_subaccounts(GtkWidget *widget, gpointer callback_data)
{
     GList *cursor;
     AccountData *parent;

     if (acc_entry_get_account(chart_setup.account_entry, &parent) || (parent == NULL))
	  return;

     for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor)) {
	  ChartAccountData *chart_account_data;
	  gchar *text[2];
	  gint row;

	  if (AccountPtr(cursor->data)->parent != parent)
	       continue;

	  chart_account_data = g_new(ChartAccountData, 1);
	  chart_account_data->account = AccountPtr(cursor->data);
	  chart_account_data->rgb = chart_setup_random_rgb();
	  
	  text[0] = chart_account_data->account->name;
	  text[1] = "";

	  row = gtk_clist_append(GTK_CLIST(chart_setup.account_list), text);
	  gtk_clist_set_row_data(GTK_CLIST(chart_setup.account_list), row, chart_account_data);
     }
}

void chart_setup_add_all_subaccounts(GtkWidget *widget, gpointer callback_data)
{
     GList *cursor;
     AccountData *parent;

     if (acc_entry_get_account(chart_setup.account_entry, &parent) || (parent == NULL))
	  return;

     for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor)) {
	  ChartAccountData *chart_account_data;
	  AccountData *check;
	  gchar *text[2];
	  gint row;
	  
	  for (check = AccountPtr(cursor->data)->parent; check != NULL; check = check->parent) 
	       if (check == parent)
		    break;

	  if (check == NULL)
	       continue;

	  chart_account_data = g_new(ChartAccountData, 1);
	  chart_account_data->account = AccountPtr(cursor->data);
	  chart_account_data->rgb = chart_setup_random_rgb();

	  text[0] = chart_account_data->account->name;
	  text[1] = "";

	  row = gtk_clist_append(GTK_CLIST(chart_setup.account_list), text);
	  gtk_clist_set_row_data(GTK_CLIST(chart_setup.account_list), row, chart_account_data);
     }
}

void chart_setup_remove_account(GtkWidget *widget, gpointer callback_data)
{
     ChartAccountData *chart_account_data;
     gint row;

     row = GPOINTER_TO_INT(GTK_CLIST(chart_setup.account_list)->selection->data);
     chart_account_data = gtk_clist_get_row_data(GTK_CLIST(chart_setup.account_list), row);
     g_free(chart_account_data);
     gtk_clist_remove(GTK_CLIST(chart_setup.account_list), row);
}

void chart_setup_clear_accounts(GtkWidget *widget, gpointer callback_data)
{
     ChartAccountData *chart_account_data;
     gint row;

     for (row = 0; row < GTK_CLIST(chart_setup.account_list)->rows; row++) {
	  chart_account_data = gtk_clist_get_row_data(GTK_CLIST(chart_setup.account_list), row);
	  g_free(chart_account_data);
     }

     gtk_clist_clear(GTK_CLIST(chart_setup.account_list));
}

void chart_setup_move_up_account(GtkWidget *widget, gpointer callback_data)
{
     ChartAccountData *chart_account_data;
     gint row;
     gchar *text[2];

     row = GPOINTER_TO_INT(GTK_CLIST(chart_setup.account_list)->selection->data);
     if (row == 0)
	  return;

     chart_account_data = gtk_clist_get_row_data(GTK_CLIST(chart_setup.account_list), row);
     gtk_clist_remove(GTK_CLIST(chart_setup.account_list), row);
     text[0] = chart_account_data->account->name;
     text[1] = "";
     row = gtk_clist_insert(GTK_CLIST(chart_setup.account_list), row-1, text);
     gtk_clist_set_row_data(GTK_CLIST(chart_setup.account_list), row, chart_account_data);
     gtk_clist_select_row(GTK_CLIST(chart_setup.account_list), row, 0);
}

void chart_setup_move_down_account(GtkWidget *widget, gpointer callback_data)
{
     ChartAccountData *chart_account_data;
     gint row;
     gchar *text[2];

     row = GPOINTER_TO_INT(GTK_CLIST(chart_setup.account_list)->selection->data);
     chart_account_data = gtk_clist_get_row_data(GTK_CLIST(chart_setup.account_list), row);
     gtk_clist_remove(GTK_CLIST(chart_setup.account_list), row);
     text[0] = chart_account_data->account->name;
     text[1] = "";
     row = gtk_clist_insert(GTK_CLIST(chart_setup.account_list), row+1, text);
     gtk_clist_set_row_data(GTK_CLIST(chart_setup.account_list), row, chart_account_data);
     gtk_clist_select_row(GTK_CLIST(chart_setup.account_list), row, 0);
}

void chart_setup_add_group(GtkWidget *widget, gpointer callback_data)
{
     ChartGroupData *chart_group_data;
     gint row;
     gchar *text[2];

     chart_group_data = g_new(ChartGroupData, 1);
     strcpy(chart_group_data->name, gtk_entry_get_text(GTK_ENTRY(chart_setup.group_name)));
     strcpy(chart_group_data->expr, gtk_entry_get_text(GTK_ENTRY(chart_setup.group_expr)));

     text[0] = chart_group_data->name;
     text[1] = chart_group_data->expr;
     row = gtk_clist_append(GTK_CLIST(chart_setup.group_list), text);
     gtk_clist_set_row_data(GTK_CLIST(chart_setup.group_list), row, chart_group_data);
}

void chart_setup_remove_group(GtkWidget *widget, gpointer callback_data)
{
     ChartGroupData *chart_group_data;
     gint row;

     row = GPOINTER_TO_INT(GTK_CLIST(chart_setup.group_list)->selection->data);
     chart_group_data = gtk_clist_get_row_data(GTK_CLIST(chart_setup.group_list), row);
     g_free(chart_group_data);
     gtk_clist_remove(GTK_CLIST(chart_setup.group_list), row);
}

void chart_setup_clear_groups(GtkWidget *widget, gpointer callback_data)
{
     ChartGroupData *chart_group_data;
     gint row;

     for (row = 0; row < GTK_CLIST(chart_setup.group_list)->rows; row++) {
	  chart_group_data = gtk_clist_get_row_data(GTK_CLIST(chart_setup.group_list), row);
	  g_free(chart_group_data);
     }

     gtk_clist_clear(GTK_CLIST(chart_setup.group_list));
}

void chart_setup_move_up_group(GtkWidget *widget, gpointer callback_data)
{
     ChartGroupData *group;
     gint row;
     gchar *text[2];

     row = GPOINTER_TO_INT(GTK_CLIST(chart_setup.group_list)->selection->data);
     if (row == 0)
	  return;

     group = gtk_clist_get_row_data(GTK_CLIST(chart_setup.group_list), row);
     gtk_clist_remove(GTK_CLIST(chart_setup.group_list), row);
     text[0] = group->name;
     text[1] = group->expr;
     row = gtk_clist_insert(GTK_CLIST(chart_setup.group_list), row-1, text);
     gtk_clist_set_row_data(GTK_CLIST(chart_setup.group_list), row, group);
     gtk_clist_select_row(GTK_CLIST(chart_setup.group_list), row, 0);
}

void chart_setup_move_down_group(GtkWidget *widget, gpointer callback_data)
{
     ChartGroupData *group;
     gint row;
     gchar *text[2];

     row = GPOINTER_TO_INT(GTK_CLIST(chart_setup.group_list)->selection->data);

     group = gtk_clist_get_row_data(GTK_CLIST(chart_setup.group_list), row);
     gtk_clist_remove(GTK_CLIST(chart_setup.group_list), row);
     text[0] = group->name;
     text[1] = group->expr;
     row = gtk_clist_insert(GTK_CLIST(chart_setup.group_list), row+1, text);
     gtk_clist_set_row_data(GTK_CLIST(chart_setup.group_list), row, group);
     gtk_clist_select_row(GTK_CLIST(chart_setup.group_list), row, 0);
}

void chart_setup_group_edit_expr(GtkWidget *widget, gpointer callback_data)
{
     gchar expr[PY_EXPRESSION_MAX_SIZE];
     strcpy(expr, gtk_entry_get_text(GTK_ENTRY(chart_setup.group_expr)));

     if (limit_dlg_execute(_("Edit group limit..."), expr))
	  gtk_entry_set_text(GTK_ENTRY(chart_setup.group_expr), expr);
}

GtkWidget *chart_setup_build_chart_type(void)
{
     chart_setup.bar_chart = gtk_radio_button_new_with_label(NULL, "Bar chart");
     gtk_widget_show(chart_setup.bar_chart);
     return chart_setup.bar_chart;
}

GtkWidget *chart_setup_build_accounts(void)
{
     GtkWidget *hbox;
     GtkWidget *vbox;
     vbox = gtk_vbox_new(FALSE, 5);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

     {
	  GtkWidget *color;

	  hbox = gtk_hbox_new(FALSE, 10);

	  chart_setup.account_entry = acc_entry_new(FALSE);
	  gtk_box_pack_start(GTK_BOX(hbox), chart_setup.account_entry, TRUE, TRUE, 0);
	  gtk_widget_show(chart_setup.account_entry);

	  /*color = gnome_color_picker_new();*/
	  color = gnome_color_picker_new();
	  gtk_box_pack_start(GTK_BOX(hbox), color, FALSE, FALSE, 0);
	  gtk_widget_show(color);	  

	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	  gtk_widget_show(hbox);
     }

     {
	  GtkWidget *add_self;
	  GtkWidget *add_children;
	  GtkWidget *add_all_children;

	  hbox = gtk_hbox_new(TRUE, 10);

	  add_self = gtk_button_new_with_label(_("Add account"));
	  gtk_signal_connect(GTK_OBJECT(add_self), "clicked", GTK_SIGNAL_FUNC(chart_setup_add_account), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), add_self, FALSE, FALSE, 0);
	  gtk_widget_show(add_self);

	  add_children = gtk_button_new_with_label(_("Add direct subaccounts"));
	  gtk_signal_connect(GTK_OBJECT(add_children), "clicked", 
			     GTK_SIGNAL_FUNC(chart_setup_add_direct_subaccounts), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), add_children, FALSE, FALSE, 0);
	  gtk_widget_show(add_children);

	  add_all_children = gtk_button_new_with_label(_("Add all subaccounts"));
	  gtk_signal_connect(GTK_OBJECT(add_all_children), "clicked", 
			     GTK_SIGNAL_FUNC(chart_setup_add_all_subaccounts), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), add_all_children, FALSE, FALSE, 0);
	  gtk_widget_show(add_all_children);
	  
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	  gtk_widget_show(hbox);
     }

     {
	  GtkWidget *separator;
	  separator = gtk_hseparator_new();
	  gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
	  gtk_widget_show(separator);
     }

     {
	  GtkWidget *move_up;
	  GtkWidget *move_down;
	  GtkWidget *remove;
	  GtkWidget *clear;

	  hbox = gtk_hbox_new(TRUE, 10);

	  move_up = gtk_button_new_with_label(_("Move up"));
	  gtk_signal_connect(GTK_OBJECT(move_up), "clicked", GTK_SIGNAL_FUNC(chart_setup_move_up_account), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), move_up, FALSE, FALSE, 0);
	  gtk_widget_show(move_up);

	  move_down = gtk_button_new_with_label(_("Move down"));
	  gtk_signal_connect(GTK_OBJECT(move_down), "clicked", GTK_SIGNAL_FUNC(chart_setup_move_down_account), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), move_down, FALSE, FALSE, 0);
	  gtk_widget_show(move_down);

	  remove = gtk_button_new_with_label(_("Remove"));
	  gtk_signal_connect(GTK_OBJECT(remove), "clicked", GTK_SIGNAL_FUNC(chart_setup_remove_account), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), remove, FALSE, FALSE, 0);
	  gtk_widget_show(remove);

	  clear = gtk_button_new_with_label(_("Clear"));
	  gtk_signal_connect(GTK_OBJECT(clear), "clicked", GTK_SIGNAL_FUNC(chart_setup_clear_accounts), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), clear, FALSE, FALSE, 0);
	  gtk_widget_show(clear);

	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	  gtk_widget_show(hbox);
     }

     {
	  gchar *titles[2] = {"Account","Color"};
	  chart_setup.account_list = gtk_clist_new_with_titles(1, titles);
	  gtk_clist_set_column_width(GTK_CLIST(chart_setup.account_list), 0, 140);
	  gtk_clist_column_titles_hide(GTK_CLIST(chart_setup.account_list));
	  gtk_widget_set_usize(chart_setup.account_list, 100, 200);
	  gtk_box_pack_start(GTK_BOX(vbox), chart_setup.account_list, TRUE, TRUE, 0);
	  gtk_widget_show(chart_setup.account_list);
     }

     {
	  GList *cursor;
	  ChartAccountData *chart_account_data;
	  for (cursor = chart_acc_list; cursor != NULL; cursor = g_list_next(cursor)) {
	       gint row;
	       gchar *text[2];

	       chart_account_data = g_new(ChartAccountData, 1);
	       chart_account_data->account = ChartAccountDataPtr(cursor->data)->account;
	       chart_account_data->rgb = ChartAccountDataPtr(cursor->data)->rgb;

	       text[0] = chart_account_data->account->name;
	       text[1] = "";
	       row = gtk_clist_append(GTK_CLIST(chart_setup.account_list), text);
	       gtk_clist_set_row_data(GTK_CLIST(chart_setup.account_list), row, chart_account_data);
	  }
     }

     gtk_widget_show(vbox);
     return vbox;
}

GtkWidget *chart_setup_build_groups(void)
{
     GtkWidget *hbox;
     GtkWidget *vbox;
     GtkWidget *label;
     vbox = gtk_vbox_new(FALSE, 5);
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

     {
	  GtkWidget *table;
	  GtkWidget *edit_button;

       	  table = gtk_table_new(3, 2, FALSE);

	  label = gtk_label_new(_("Label:"));
	  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);
	  gtk_widget_show(label);

	  chart_setup.group_name = gtk_entry_new();
	  gtk_table_attach(GTK_TABLE(table), chart_setup.group_name, 1, 3, 0, 1, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	  gtk_widget_show(chart_setup.group_name);

	  label = gtk_label_new(_("Expr:"));
	  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, 0, 3, 3);
	  gtk_widget_show(label);

	  chart_setup.group_expr = gtk_entry_new();
	  gtk_table_attach(GTK_TABLE(table), chart_setup.group_expr, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	  gtk_widget_show(chart_setup.group_expr);

	  edit_button = gtk_button_new_with_label(_("Edit..."));
	  gtk_signal_connect(GTK_OBJECT(edit_button), "clicked", GTK_SIGNAL_FUNC(chart_setup_group_edit_expr), NULL);
	  gtk_table_attach(GTK_TABLE(table), edit_button, 2, 3, 1, 2, GTK_FILL, 0, 3, 3);
	  gtk_widget_show(edit_button);

	  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
	  gtk_widget_show(table);
     }

     {
	  GtkWidget *separator;
	  GtkWidget *add;
	  GtkWidget *move_up;
	  GtkWidget *move_down;
	  GtkWidget *remove;
	  GtkWidget *clear;

	  hbox = gtk_hbox_new(TRUE, 10);

	  add = gtk_button_new_with_label(_("Add"));
	  gtk_signal_connect(GTK_OBJECT(add), "clicked", 
			     GTK_SIGNAL_FUNC(chart_setup_add_group), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), add, FALSE, FALSE, 0);
	  gtk_widget_show(add);

	  separator = gtk_vseparator_new();
	  gtk_box_pack_start(GTK_BOX(hbox), separator, FALSE, FALSE, 0);
	  gtk_widget_show(separator);

	  move_up = gtk_button_new_with_label(_("Move up"));
	  gtk_signal_connect(GTK_OBJECT(move_up), "clicked", GTK_SIGNAL_FUNC(chart_setup_move_up_group), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), move_up, FALSE, FALSE, 0);
	  gtk_widget_show(move_up);

	  move_down = gtk_button_new_with_label(_("Move down"));
	  gtk_signal_connect(GTK_OBJECT(move_down), "clicked", GTK_SIGNAL_FUNC(chart_setup_move_down_group), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), move_down, FALSE, FALSE, 0);
	  gtk_widget_show(move_down);

	  remove = gtk_button_new_with_label(_("Remove"));
	  gtk_signal_connect(GTK_OBJECT(remove), "clicked", GTK_SIGNAL_FUNC(chart_setup_remove_group), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), remove, FALSE, FALSE, 0);
	  gtk_widget_show(remove);
	  
	  clear = gtk_button_new_with_label(_("Clear"));
	  gtk_signal_connect(GTK_OBJECT(clear), "clicked", GTK_SIGNAL_FUNC(chart_setup_clear_groups), NULL);
	  gtk_box_pack_start(GTK_BOX(hbox), clear, FALSE, FALSE, 0);
	  gtk_widget_show(clear);

	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	  gtk_widget_show(hbox);
     }

     {
	  gchar *titles[2] = {_("Label"), _("Expression")};
	  chart_setup.group_list = gtk_clist_new_with_titles(2, titles);
	  gtk_widget_set_usize(chart_setup.group_list, 200, 200);
	  gtk_clist_set_column_width(GTK_CLIST(chart_setup.account_list), 0, 100);
	  gtk_box_pack_start(GTK_BOX(vbox), chart_setup.group_list, TRUE, TRUE, 0);
	  gtk_widget_show(chart_setup.group_list);
     }

     {
	  GList *cursor;
	  ChartGroupData *chart_group_data;

	  if (!((g_list_length(chart_group_list) == 1) && 
		(strcmp(ChartGroupDataPtr(chart_group_list->data)->expr, "") == 0))) {

	       for (cursor = chart_group_list; cursor != NULL; cursor = g_list_next(cursor)) {
		    gint row;
		    gchar *text[2];

		    chart_group_data = g_new(ChartGroupData, 1);
		    strcpy(chart_group_data->name, ChartGroupDataPtr(cursor->data)->name);
		    strcpy(chart_group_data->expr, ChartGroupDataPtr(cursor->data)->expr);

		    text[0] = chart_group_data->name;
		    text[1] = chart_group_data->expr;
		    row = gtk_clist_append(GTK_CLIST(chart_setup.group_list), text);
		    gtk_clist_set_row_data(GTK_CLIST(chart_setup.group_list), row, chart_group_data);
	       }
	  }
     }

     gtk_widget_show(vbox);
     return vbox;
}

GtkWidget *chart_setup_build_notebook(void)
{
    GtkWidget *notebook;

    notebook = gtk_notebook_new();

    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), chart_setup_build_chart_type(), gtk_label_new(_("Chart type")));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), chart_setup_build_accounts(), gtk_label_new(_("Accounts")));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), chart_setup_build_groups(), 
			     gtk_label_new(_("Groups")));
     
    gtk_widget_show(notebook);

    return notebook;
}

void chart_setup_build(void)
{
    chart_setup.dialog = gnome_dialog_new("Setup chart...", GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(chart_setup.dialog)->vbox), chart_setup_build_notebook(), TRUE, TRUE, 0);
}

void chart_setup_main(void)
{
     gint row;

     chart_done();
     chart_init();

     for (row = 0; row < GTK_CLIST(chart_setup.account_list)->rows; row++) {
	  ChartAccountData *chart_account = gtk_clist_get_row_data(GTK_CLIST(chart_setup.account_list), row);
	  chart_acc_list = g_list_prepend(chart_acc_list, chart_account);
     }
     chart_acc_list = g_list_reverse(chart_acc_list);

     for (row = 0; row < GTK_CLIST(chart_setup.group_list)->rows; row++) {
	  ChartGroupData *group = gtk_clist_get_row_data(GTK_CLIST(chart_setup.group_list), row);
	  chart_group_list = g_list_prepend(chart_group_list, group);
     }
     chart_group_list = g_list_reverse(chart_group_list);

     if (chart_group_list == NULL) {
	  ChartGroupData *group;

	  group = g_new(ChartGroupData, 1);
	  strcpy(group->name, "");
	  strcpy(group->expr, "");

	  chart_group_list = g_list_prepend(chart_group_list, group);
     }
}

gboolean chart_setup_execute(void)
{
     gint ret;

     chart_setup_build();
     ret = gnome_dialog_run(GNOME_DIALOG(chart_setup.dialog));

     switch (ret) {
     case 0:		/* OK */
	  chart_setup_main();
     case 1:		/* Cancel */
	  gnome_dialog_close(GNOME_DIALOG(chart_setup.dialog));
     case -1:		/* Esc or "closed window" */
	  chart_setup_clear_accounts(NULL, NULL);
	  chart_setup_clear_groups(NULL, NULL);
	  break;
     default:
	  g_assert_not_reached();
     }

     return ret == 0; /* Ok */
}






