#include <config.h>

#include <ctype.h>
#include <langinfo.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include <Python.h>

#include "misc.h"

#include "py_module.h"


TransactionData *py_module_transaction = NULL;

PyObject *py_module_amount(PyObject *self, PyObject *args);
PyObject *py_module_from(PyObject *self, PyObject *args);
PyObject *py_module_to(PyObject *self, PyObject *args);
PyObject *py_module_date(PyObject *self, PyObject *args);
PyObject *py_module_day(PyObject *self, PyObject *args);
PyObject *py_module_month(PyObject *self, PyObject *args);
PyObject *py_module_year(PyObject *self, PyObject *args);
PyObject *py_module_desc(PyObject *self, PyObject *args);
PyObject *py_module_parents(PyObject *self, PyObject *args);

PyObject *py_module_register_func(PyObject *self, PyObject *args);

PyObject *py_module_date_value(PyObject *self, PyObject *args);

PyObject *py_module_get_value(PyObject *self, PyObject *args);

static PyMethodDef EmmaMethods[] = {
     { "amount", py_module_amount, METH_VARARGS },
     { "acc_from", py_module_from, METH_VARARGS },
     { "acc_to", py_module_to, METH_VARARGS },
     { "date", py_module_date, METH_VARARGS },
     { "desc", py_module_desc, METH_VARARGS },
     { "parents", py_module_parents, METH_VARARGS },

     { "register_func", py_module_register_func, METH_VARARGS },

     { "date_val", py_module_date_value, METH_VARARGS },

     { "__badhack__", py_module_get_value, METH_VARARGS },

     { NULL, NULL }
};

void py_module_import_module(const gchar *module_name)
{
     gchar cmd[1024] = "import ";
     strcat(cmd, module_name);
     PyRun_SimpleString(cmd);
}

void py_module_include_file(gchar *filename, const gboolean verbose_failure)
{
     if (!access(filename, F_OK)) { 	/* file exist ? */
	  FILE *stream;
	  /* g_print("Including python source-file %s\n",filename); */
	  stream = fopen(filename, "r");
	  PyRun_SimpleFile(stream, filename);
	  fclose(stream);
     } else
	  if (verbose_failure)
	       g_print("Coudn't access python source-file %s\n", filename);
}

void py_module_include_appriopriate_files(void)
{
     gchar tmp[1024];

     strcpy(tmp, g_get_home_dir());
     strcat(tmp, "/.emma.py");
     py_module_include_file(tmp, FALSE);
}

void py_module_init(void)
{
     py_module_func_desc = NULL;

     Py_Initialize();
     PyImport_AddModule("emma");
     Py_InitModule("emma", EmmaMethods);
     PyRun_SimpleString("import emma");
     
     py_module_include_appriopriate_files();

     strcpy(limit_text, "");
}

void py_module_done(void)
{
     GList *cursor;

     for (cursor = py_module_func_desc; cursor != NULL; cursor = g_list_next(cursor)) {
	  PyModuleFuncDesc *func_desc = PyModuleFuncDescPtr(cursor->data);
	  g_free(func_desc->name);
	  g_free(func_desc->desc);
     }
     g_list_free(py_module_func_desc);

     Py_Finalize();
}

void py_module_set_trans(TransactionData *trans)
{
     py_module_transaction = trans;
}

gchar py_module_exec_str[PY_EXPRESSION_MAX_SIZE];
gboolean py_module_ret_val;

void py_module_compile_str(const gchar *str)
{
     if (only_white_space(str))
	  strcpy(py_module_exec_str, "emma.__badhack__(1)");
     else {
	  strcpy(py_module_exec_str, "emma.__badhack__(");
	  strcat(py_module_exec_str, str);
	  strcat(py_module_exec_str, ")");
     }
}

gboolean py_module_eval_code(void)
{
     PyRun_SimpleString(py_module_exec_str);
     return py_module_ret_val;
}

PyObject *py_module_get_value(PyObject *self, PyObject *args)
{
     PyObject *res = NULL;
     gint tmp;
     if (PyArg_ParseTuple(args, "i:parents", &tmp)) {
	  Py_INCREF(Py_None);
	  py_module_ret_val = (tmp != 0);
	  res = Py_None;
     }
     return res;
}

PyObject *py_module_amount(PyObject *self, PyObject *args)
{
     return Py_BuildValue("d",(double)(py_module_transaction->amount));
}

PyObject *py_module_from(PyObject *self, PyObject *args)
{
     return Py_BuildValue("s",py_module_transaction->acc_from->name);
}

PyObject *py_module_to(PyObject *self, PyObject *args)
{
     return Py_BuildValue("s",py_module_transaction->acc_to->name);
}

PyObject *py_module_date(PyObject *self, PyObject *args)
{
     struct tm tmp;
     g_date_to_struct_tm(&(py_module_transaction->date), &tmp);
     return Py_BuildValue("d",(double)mktime(&tmp));
}

PyObject *py_module_day(PyObject *self, PyObject *args)
{
     return Py_BuildValue("d",(double)g_date_day(&(py_module_transaction->date)));
}

PyObject *py_module_month(PyObject *self, PyObject *args)
{
     return Py_BuildValue("d",(double)g_date_month(&(py_module_transaction->date)));
}

PyObject *py_module_year(PyObject *self, PyObject *args)
{
     return Py_BuildValue("d",(double)g_date_year(&(py_module_transaction->date)));
}

PyObject *py_module_desc(PyObject *self, PyObject *args)
{
     return Py_BuildValue("s",py_module_transaction->desc);
}

PyObject *py_module_parents(PyObject *self, PyObject *args)
{
     PyObject *res = NULL;
     gchar *acc_name;

     if (PyArg_ParseTuple(args, "s:parents", &acc_name)) {
	  GList *cursor;
	  AccountData *parent = NULL;

	  for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor)) 
	       if (strcmp(acc_name, AccountPtr(cursor->data)->name) == 0) {
		    parent = AccountPtr(cursor->data)->parent;
		    break;
	       }		

	  res = PyList_New(0);
	  while (parent != NULL) {
	       PyList_Append(res, Py_BuildValue("s",parent->name));
	       parent = parent->parent;
	  }
     }
     
     return res;
}

PyObject *py_module_date_value(PyObject *self, PyObject *args)
{
     PyObject *date_val = NULL;
     gchar *date_str;

     if (PyArg_ParseTuple(args, "s:date_val", &date_str)) {
	  GDate date;
	  struct tm tmp;
	  tmp.tm_mday = 0;
	  strptime(date_str, nl_langinfo(D_FMT), &tmp);
	  g_date_set_dmy(&date, tmp.tm_mday, tmp.tm_mon+1, tmp.tm_year+1900);
	  g_date_to_struct_tm(&date, &tmp);
	  date_val = Py_BuildValue("d",(double)mktime(&tmp));
     }

     return date_val;
}

PyObject *py_module_register_func(PyObject *self, PyObject *args)
{
     PyObject *value = NULL;
     gchar *name, *desc;

     if (PyArg_ParseTuple(args, "ss:register_func", &name, &desc)) {
	  PyModuleFuncDesc *new_func_desc;
	  
	  new_func_desc = g_new(PyModuleFuncDesc, 1);
	  new_func_desc->name = g_strdup(name);
	  new_func_desc->desc = g_strdup(desc);
	  py_module_func_desc = g_list_prepend(py_module_func_desc, new_func_desc);

	  Py_INCREF(Py_None);
	  value = Py_None;
     }

     return value;
}

