#include <config.h>

#include <gnome.h>
#include "gnome-ucombo.h"
#include "acc_entry.h"
#include "date_entry.h"

#include <glib.h>

#include "data.h"
#include "interface.h"
#include "misc.h"

#include "schedule.h"

#include "schedule_dlg.h"

struct _ScheduleEdit {
     GtkWidget *dialog;

     GtkWidget *from, *to;
     GtkWidget *amount;
     GtkWidget *date;
     GtkWidget *desc;
     GtkWidget *frequency;
     GtkWidget *prompt;
 
} schedule_dlg;

/* This really should be in GTK+ (copied from gnome-terminal)
 */
static gint
option_menu_get_history (GtkOptionMenu *option_menu)
{
	GtkWidget *active_widget;
	
	g_return_val_if_fail (GTK_IS_OPTION_MENU (option_menu), -1);
	
	active_widget = gtk_menu_get_active (GTK_MENU (option_menu->menu));

	if (active_widget)
		return g_list_index (GTK_MENU_SHELL (option_menu->menu)->children,
				     active_widget);
	else
		return -1;
}

void schedule_dlg_build_dialog(const gchar *title)
{
    schedule_dlg.dialog = gnome_dialog_new(title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(schedule_dlg.dialog), GTK_WINDOW(interface.application));

    {
        GtkWidget *table;
        GtkWidget *menu;
        GtkWidget *menuitem;
        GSList	  *group;
	gint i;
	
	
	table = gtk_table_new(2, 7, FALSE);

	/* insert entry widgets */
	{
	    schedule_dlg.amount = gtk_entry_new();
	    gtk_widget_show(schedule_dlg.amount);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.amount, 1, 2, 0, 1, GTK_FILL, 0, 3, 3);

	    schedule_dlg.from = acc_entry_new(FALSE);
	    gtk_widget_show(schedule_dlg.from);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.from, 1, 2, 1, 2, GTK_FILL, 0, 3, 3);

	    schedule_dlg.to = acc_entry_new(FALSE);
	    gtk_widget_show(schedule_dlg.to);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.to, 1, 2, 2, 3, GTK_FILL, 0, 3, 3);

	    schedule_dlg.date = date_entry_new();
	    gtk_widget_show(schedule_dlg.date);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.date, 1, 2, 3, 4, GTK_FILL, 0, 3, 3);

	    schedule_dlg.desc = gnome_entry_new("EmmaTransactionEditHistoryId");
	    gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(schedule_dlg.desc)->entry), SCHEDULE_DESC_SIZE-1);
	    gtk_widget_show(schedule_dlg.desc);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.desc, 1, 2, 4, 5, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	    
	    schedule_dlg.frequency = gtk_option_menu_new();
	    menu = gtk_menu_new();
	    group = NULL;
	    i = 0;
	    while (frequency_option_text[i] != NULL){ 
	        menuitem = gtk_menu_item_new_with_label(_(frequency_option_text[i]));
	        i++;
	        gtk_menu_append(GTK_MENU(menu), menuitem);
	        gtk_widget_show(menuitem);
	    }
	    gtk_option_menu_set_menu(GTK_OPTION_MENU(schedule_dlg.frequency), menu);

	    gtk_widget_show(schedule_dlg.frequency);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.frequency, 1, 2, 5, 6, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	    
	    schedule_dlg.prompt = gtk_check_button_new_with_label (_("Prompt before entering"));
	    gtk_widget_show(schedule_dlg.prompt);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.prompt, 1, 2, 6, 7, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	    
	}

	/* insert labels */
	{
	    GtkWidget *label;

	    label = gtk_label_new(_("Amount:"));
	    gtk_widget_show(label);
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("From:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("To:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("First Date:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("Description:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, 0, 0, 3, 3);
	    
	    gtk_widget_show(label = gtk_label_new(_("Frequency:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6, 0, 0, 3, 3);	    
	}

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(schedule_dlg.dialog)->vbox), table, TRUE, TRUE, 0);
	gtk_widget_show(table);
    }
}

void schedule_dlg_preset_values(GList *schedule_selection)
{
     GList *cursor;
     ScheduleData *schedule, *schedule2;
     int amount_diff = FALSE, from_diff = FALSE, to_diff = FALSE, date_diff = FALSE, desc_diff = FALSE, frequency_diff = FALSE, prompt_diff = FALSE;

     for (cursor = schedule_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	  if (cursor->prev == NULL)
	       continue;

	  schedule = SchedulePtr(cursor->data);
	  schedule2 = SchedulePtr(cursor->prev->data);

	  amount_diff |= (schedule->amount != schedule2->amount);
	  from_diff |= (schedule->acc_from != schedule2->acc_from);
	  to_diff |= (schedule->acc_to != schedule2->acc_to);
	  date_diff |= g_date_compare(&(schedule->date), &(schedule2->date));
	  desc_diff |= strcmp(schedule->desc, schedule2->desc);
	  frequency_diff |= (schedule->frequency != schedule2->frequency);
	  prompt_diff |= (schedule->prompt != schedule2->prompt);
     }

     schedule = SchedulePtr(schedule_selection->data);

     if (amount_diff)
	  gtk_entry_set_text(GTK_ENTRY(schedule_dlg.amount), _("<don't change>"));
     else {
	  char text[16];
	  sprintf(text, "%.2f", schedule->amount);
	  gtk_entry_set_text(GTK_ENTRY(schedule_dlg.amount), text);
     }

     if (from_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry), _("<don't change>"));
     else 
	  if (schedule->acc_from == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry), schedule->acc_from->name);
     
     if (to_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry), _("<don't change>"));
     else 
	  if (schedule->acc_to == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry), schedule->acc_to->name);

     if (date_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.date)->entry), _("<don't change>"));
     else
	  date_entry_set_date(schedule_dlg.date, &(schedule->date));

     if (desc_diff)
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(schedule_dlg.desc))), _("<don't change>"));
     else
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(schedule_dlg.desc))), schedule->desc);
 
    gtk_option_menu_set_history(GTK_OPTION_MENU(schedule_dlg.frequency), schedule->frequency);
    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(schedule_dlg.prompt), schedule->prompt ? 1 : 0);
}

int schedule_dlg_main(GList * schedule_selection)
{
    GList *cursor;
    GtkWidget *menu;
    gboolean chg_amount, chg_from, chg_to, chg_date, chg_desc;

    gdouble amount_val;
    AccountData *from_val, *to_val;
    GDate date_val;
    gchar *description_val = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(schedule_dlg.desc)->entry));

    /* if description is empty or only white space than change it to ``-'' */
    if (only_white_space(description_val))
	 strcpy(description_val, "-");

    /* see if there are any entries to be unchanged */
    chg_amount = strcmp(gtk_entry_get_text(GTK_ENTRY(schedule_dlg.amount)), _("<don't change>"));
    chg_from = strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry)), _("<don't change>"));
    chg_to = strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry)), _("<don't change>"));
    chg_date = strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.date)->entry)), _("<don't change>"));
    chg_desc = strcmp(gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(schedule_dlg.desc)))), 
		       _("<don't change>"));

    /* check syntax (everything readable?) */
    if (chg_amount) {
	 /* woops - not fully implemented yet (i18n) */
	 sscanf(gtk_entry_get_text(GTK_ENTRY(schedule_dlg.amount)), "%lf", &amount_val);
    }

    if ((chg_from && (acc_entry_get_account(schedule_dlg.from, &from_val) || (from_val == NULL))) ||
	(chg_to && (acc_entry_get_account(schedule_dlg.to, &to_val) || (to_val == NULL))) ||
	(chg_date && date_entry_get_date(schedule_dlg.date, &date_val)))
	 return FALSE;

    /* and now we can actually change schedules on the list */
    for (cursor = schedule_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	ScheduleData *schedule = SchedulePtr(cursor->data);

	if (chg_amount)
	    schedule->amount = amount_val;
	if (chg_from)
	    schedule->acc_from = from_val;
	if (chg_to)
	    schedule->acc_to = to_val;
	if (chg_date)
	     schedule->date = date_val;
	if (chg_desc) 
	     strcpy(schedule->desc, description_val);

	schedule->frequency = option_menu_get_history(GTK_OPTION_MENU(schedule_dlg.frequency));

	schedule->prompt = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(schedule_dlg.prompt));
    }

    return TRUE;
}

gboolean schedule_dlg_execute(const gchar *title, GList * schedule_selection)
{
    schedule_dlg_build_dialog(title);
    schedule_dlg_preset_values(schedule_selection);

    while (TRUE) {
	int ret;

	ret = gnome_dialog_run(GNOME_DIALOG(schedule_dlg.dialog));
	switch (ret) {
	case 0:		/* OK */
	    if (schedule_dlg_main(schedule_selection)) {
		gnome_dialog_close(GNOME_DIALOG(schedule_dlg.dialog));
		return TRUE;
	    } else {
		g_print("You entered wrong data!!!\n");
		/* and appriopriate message box/dialog */
	    }
	    break;
	case 1:		/* Cancel */
	    gnome_dialog_close(GNOME_DIALOG(schedule_dlg.dialog));
	case -1:		/* Esc or "closed window" */
	    return FALSE;
	}
    };
}

