#include <config.h>

#include <gnome.h>
#include "gnome-ucombo.h"
#include "acc_entry.h"
#include "date_entry.h"

#include <glib.h>

#include "data.h"
#include "interface.h"
#include "misc.h"

#include "transaction_dlg.h"


struct _TransactionEdit {
     GtkWidget *dialog;

     GtkWidget *from, *to;
     GtkWidget *amount;
     GtkWidget *date;
     GtkWidget *desc;
} transaction_dlg;

void transaction_dlg_build_dialog(const gchar *title)
{
    transaction_dlg.dialog = gnome_dialog_new(title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(transaction_dlg.dialog), GTK_WINDOW(interface.application));

    {
	GtkWidget *table;
	table = gtk_table_new(2, 5, FALSE);

	/* insert entry widgets */
	{
	    transaction_dlg.amount = gtk_entry_new();
	    gtk_widget_show(transaction_dlg.amount);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.amount, 1, 2, 0, 1, GTK_FILL, 0, 3, 3);

	    transaction_dlg.from = acc_entry_new(FALSE);
	    gtk_widget_show(transaction_dlg.from);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.from, 1, 2, 1, 2, GTK_FILL, 0, 3, 3);

	    transaction_dlg.to = acc_entry_new(FALSE);
	    gtk_widget_show(transaction_dlg.to);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.to, 1, 2, 2, 3, GTK_FILL, 0, 3, 3);

	    transaction_dlg.date = date_entry_new();
	    gtk_widget_show(transaction_dlg.date);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.date, 1, 2, 3, 4, GTK_FILL, 0, 3, 3);

	    transaction_dlg.desc = gnome_entry_new("EmmaTransactionEditHistoryId");
	    gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(transaction_dlg.desc)->entry), TRANSACTION_DESC_SIZE-1);
	    gtk_widget_show(transaction_dlg.desc);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.desc, 1, 2, 4, 5, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	}

	/* insert labels */
	{
	    GtkWidget *label;

	    label = gtk_label_new(_("Amount:"));
	    gtk_widget_show(label);
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("From:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("To:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("Date:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("Description:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, 0, 0, 3, 3);
	}

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(transaction_dlg.dialog)->vbox), table, TRUE, TRUE, 0);
	gtk_widget_show(table);
    }
}

void transaction_dlg_preset_values(GList *trans_selection)
{
     GList *cursor;
     TransactionData *trans, *trans2;
     int amount_diff = FALSE, from_diff = FALSE, to_diff = FALSE, date_diff = FALSE, desc_diff = FALSE;

     for (cursor = trans_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	  if (cursor->prev == NULL)
	       continue;

	  trans = TransactionPtr(cursor->data);
	  trans2 = TransactionPtr(cursor->prev->data);

	  amount_diff |= (trans->amount != trans2->amount);
	  from_diff |= (trans->acc_from != trans2->acc_from);
	  to_diff |= (trans->acc_to != trans2->acc_to);
	  date_diff |= g_date_compare(&(trans->date), &(trans2->date));
	  desc_diff |= strcmp(trans->desc, trans2->desc);
     }

     trans = TransactionPtr(trans_selection->data);

     if (amount_diff)
	  gtk_entry_set_text(GTK_ENTRY(transaction_dlg.amount), _("<don't change>"));
     else {
	  char text[16];
	  sprintf(text, "%.2f", trans->amount);
	  gtk_entry_set_text(GTK_ENTRY(transaction_dlg.amount), text);
     }

     if (from_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry), _("<don't change>"));
     else 
	  if (trans->acc_from == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry), trans->acc_from->name);
     
     if (to_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry), _("<don't change>"));
     else 
	  if (trans->acc_to == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry), trans->acc_to->name);

     if (date_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.date)->entry), _("<don't change>"));
     else
	  date_entry_set_date(transaction_dlg.date, &(trans->date));

     if (desc_diff)
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(transaction_dlg.desc))), _("<don't change>"));
     else
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(transaction_dlg.desc))), trans->desc);

}

int transaction_dlg_main(GList * trans_selection)
{
    GList *cursor;
    gboolean chg_amount, chg_from, chg_to, chg_date, chg_desc;

    gdouble amount_val;
    AccountData *from_val, *to_val;
    GDate date_val;
    gchar *description_val = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(transaction_dlg.desc)->entry));

    /* if description is empty or only white space than change it to ``-'' */
    if (only_white_space(description_val))
	 strcpy(description_val, "-");

    /* see if there are any entries to be unchanged */
    chg_amount = strcmp(gtk_entry_get_text(GTK_ENTRY(transaction_dlg.amount)), _("<don't change>"));
    chg_from = strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry)), _("<don't change>"));
    chg_to = strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry)), _("<don't change>"));
    chg_date = strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.date)->entry)), _("<don't change>"));
    chg_desc = strcmp(gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(transaction_dlg.desc)))), 
		       _("<don't change>"));
    g_print("Chg status : %d %d %d %d %d\n",chg_amount,chg_from,chg_to,chg_date,chg_desc);

    /* check syntax (everything readable?) */
    if (chg_amount) {
	 /* woops - not fully implemented yet (i18n) */
	 sscanf(gtk_entry_get_text(GTK_ENTRY(transaction_dlg.amount)), "%lf", &amount_val);
    }

    if ((chg_from && (acc_entry_get_account(transaction_dlg.from, &from_val) || (from_val == NULL))) ||
	(chg_to && (acc_entry_get_account(transaction_dlg.to, &to_val) || (to_val == NULL))) ||
	(chg_date && date_entry_get_date(transaction_dlg.date, &date_val)))
	 return FALSE;

    /* and now we can actually change transactions on the list */
    for (cursor = trans_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	TransactionData *transaction = TransactionPtr(cursor->data);

	if (chg_amount)
	    transaction->amount = amount_val;
	if (chg_from)
	    transaction->acc_from = from_val;
	if (chg_to)
	    transaction->acc_to = to_val;
	if (chg_date)
	     transaction->date = date_val;
	if (chg_desc) 
	     strcpy(transaction->desc, description_val);
    }

    return TRUE;
}

gboolean transaction_dlg_execute(const gchar *title, GList * trans_selection)
{
    transaction_dlg_build_dialog(title);
    transaction_dlg_preset_values(trans_selection);

    while (TRUE) {
	int ret;

	ret = gnome_dialog_run(GNOME_DIALOG(transaction_dlg.dialog));
	switch (ret) {
	case 0:		/* OK */
	    if (transaction_dlg_main(trans_selection)) {
		gnome_dialog_close(GNOME_DIALOG(transaction_dlg.dialog));
		return TRUE;
	    } else {
		g_print("You entered wrong data!!!\n");
		/* and appriopriate message box/dialog */
	    }
	    break;
	case 1:		/* Cancel */
	    gnome_dialog_close(GNOME_DIALOG(transaction_dlg.dialog));
	case -1:		/* Esc or "closed window" */
	    return FALSE;
	}
    };
}
